#ifndef LIBGRAPH_COMSCHEDULE_IS_DEFINED
#define LIBGRAPH_COMSCHEDULE_IS_DEFINED

#include <kbps.h>
#include <CommunicationScheduling.h> 

class paco_comSchedule_libgraph : public paco_comSchedule
{
protected:
  kbps_approximation* _kbps;

  unsigned int* _tmp_idx; // helper array for phase,id -> k
  inline unsigned int _idx(const unsigned phase, const unsigned id)
  {
    return _tmp_idx[phase]+id;
  }

public:
  paco_comSchedule_libgraph(kbps_approximation* k) : _kbps(k), _tmp_idx(NULL) {}

  kbps_approximation* get_kbps() const { return _kbps; }

  void build_idx()
  {
    if (_tmp_idx) {
      delete[] _tmp_idx;
    }
    
    _tmp_idx = new unsigned int[this->getNbPhases()];
    _tmp_idx[0]=0;
    for(unsigned i=1;i<getNbPhases(); i++)
      _tmp_idx[i]=_tmp_idx[i-1]+getElementPhase(i-1);
  }

  unsigned int getNbPhases() {
    return _kbps->infophases[0];
  }
  unsigned int getElementPhase(const unsigned phase)
  {
    return _kbps->infophases[phase+1];
  }
  unsigned int getSrc(const unsigned phase, const unsigned id)
  {
    return _kbps->comms[_idx(phase, id)].src;
  }
  unsigned int getDst(const unsigned phase, const unsigned id)
  {
    return _kbps->comms[_idx(phase, id)].dst;
  }
  unsigned int getSize(const unsigned phase, const unsigned id)
  {
    return _kbps->comms[_idx(phase, id)].weight;
  }

  void* getInfo(const unsigned phase, const unsigned id)
  {
    return _kbps->comms[_idx(phase, id)].info;    
  }

};

class paco_comScheduling_libgraph : public paco_comScheduling 
{
public:

  paco_comScheduling_libgraph();
  virtual ~paco_comScheduling_libgraph();

  void set_beta(unsigned int beta); // reset all schedule memory  (but no communication memory)
  void set_k(unsigned int k); // reset all schedule memory (but no communication memory)
  
  unsigned int get_beta();
  unsigned int get_k();

  paco_comSchedule* computeScheduling(unsigned rank, 
				      PaCO::PacoTopology_t* srctopo, PaCO::PacoTopology_t* dsttopo,
				      unsigned **com_matrix, void** info_vector[]);
  
protected:
  /** create and initialize a graph */
  virtual void initializeBigraph(PaCO::PacoTopology_t* srctopo, PaCO::PacoTopology_t* dsttopo); 

  unsigned int _beta;
  unsigned int _k;

  bigraph * _g; // MUST be configured only by configureLibGraph
  node ** _client_nodes;
  node ** _server_nodes;

};

class paco_libgraph_fabrique : virtual public paco_fabrique_comScheduling
{
 public:
   paco_comScheduling *create_comScheduling() {
    return new paco_comScheduling_libgraph();
  }
};

#endif
